@@ -48,30 +48,50 @@ sub print_lexicals {
   my ($self, $lexicals, $frame_index) = @_;
   my $output = '<ul>';
   local $Data::Dumper::Terse = 1;
-
+  local $Data::Dumper::Indent = 1;
+    
   foreach my $var (sort keys %$lexicals) {
-    my $val = Dumper(${ $lexicals->{$var} });
+    my $val;
+    if(ref $lexicals->{$var} eq 'REF' || ref $lexicals->{$var} eq 'SCALAR') {
+      $val = Dumper(${ $lexicals->{$var} });
+    } else {
+      $val = Dumper( $lexicals->{$var} );
+    }
+    chomp $val;
     $val = escapeHTML($val);
-    my $out;
     my $edit_link = $self->request->callback_link(
-      Edit => sub {
+      "$var" => sub {
         my $save_button = $self->request->callback_submit(
           Save => sub {
             my $val = $self->param('blah');
-            ${ $lexicals->{$var} } = eval($val);
+            $val = eval($val);
+            unless ($@) {
+              if(ref $lexicals->{$var} eq 'REF' || ref $lexicals->{$var} eq 'SCALAR') {
+                ${ $lexicals->{$var} } = $val;
+              } elsif(ref $lexicals->{$var} eq 'ARRAY') {
+                @{ $lexicals->{$var} } = @{ $val };
+              } elsif(ref $lexicals->{$var} eq 'HASH') {
+                %{ $lexicals->{$var} } = %{ $val };
+              }
+            }
           }
         );
         $self->{output} = qq{
           <div class=dialog id=stacktrace title='Stacktrace'>
             $var = <textarea name=blah style="width: 100%; height: 80%">$val</textarea><br>
             $save_button
+            <input type=submit name=cancel value="Cancel">
           </div>
         };
       }
     );
-    $out ||= "<li><pre>$var = $val</pre>$edit_link</li>";
-    #$out ||= "<li><pre>$var = $val</pre></li>";
-    $output .= "<li>$out</li>";
+    $output .= qq{
+      <li>
+        <table border=0>
+          <tr><td valign=top>$edit_link =</td>
+          <td valign=top><pre>$val</pre></td>
+        </table>
+      </li>};
   }
 
   $output .= "</ul>";
